from padlib import particle_system

import pygame
import random
import copy
from collections import deque

from flower import Flower
from grid import Grid, Node
from day import Day
#from sun import Sun
from mutate_color import MutateColor
from sun_sprite import SunSprite
from cloud_sprite import CloudSprite
from bug_sprite import BugSprite

import game_screen

#States
WEATHER_EFFECT_START = 1
CHANGE_DNA = 2
SPAWN_PLANTS = 3
BUG_BONUS = 4
WEATHER_EFFECT_END = 5
DAY_CHANGES = 6

#Players
PLAYER = 1
RIVAL = 2

#AI
CHOOSE_ROTATE_LEFT = 0
CHOOSE_ROTATE_RIGHT = 1
CHOOSE_SWAP_1 = 2
CHOOSE_SWAP_2 = 3
CHOOSE_RANDOM = 4

SUN_START_POS_X = 300
SUN_START_POS_Y = 90

SUN_END_POS_X = 300
SUN_END_POS_Y = 300

CLOUD_START_POS_X = 300
CLOUD_START_POS_Y = 80

CLOUD_END_POS_X = -150
CLOUD_END_POS_Y = 80

NUCLEOBASES = ['R', 'G', 'B' ]

class State(object):
	def __init__(self, game):
		self.game = game
		self.clock = pygame.time.Clock()
		self.is_completed = False
		self.days = deque()
		self.grid = Grid()
		self.fps = 30
		self.numTurns = 0

		self.state = CHANGE_DNA

		self.flowers = pygame.sprite.OrderedUpdates()

		#Player
		player_starting_posX = 0
		player_starting_posY = 0

		player_start_flower = Flower( self, PLAYER )
		player_start_flower.selected = True
		player_start_flower.randomize_dna()
		self.grid.setNode(player_starting_posX, player_starting_posY, player_start_flower )
		self.next_flower_dna = copy.copy(player_start_flower.dna)
		self.flowers.add( player_start_flower )
		width, height = Flower.NORM_PLANT_SIZE
		player_start_flower.scaleTo( width, height, 1, self.fps )

		self.currentPlayerFlower = player_start_flower

		#Rival
		rival_starting_posX = self.grid.num_rows-1
		rival_starting_posY = self.grid.num_columns-1

		rival_start_flower = Flower(self, RIVAL)
		rival_start_flower.randomize_dna()
		self.grid.setNode(rival_starting_posX, rival_starting_posY, rival_start_flower )
		self.rival_next_flower_dna = copy.copy(rival_start_flower.dna)
		self.flowers.add( rival_start_flower )
		width, height = Flower.NORM_PLANT_SIZE
		rival_start_flower.scaleTo( width, height, 1, self.fps )

		self.currentRivalFlower = rival_start_flower

		#Weather
		self.sun_sprites = pygame.sprite.OrderedUpdates()
		self.sun = SunSprite( MutateColor(0,0,0).sequence("RRR"), (SUN_START_POS_X, SUN_START_POS_Y))
		self.rain = CloudSprite( MutateColor(0,0,0).sequence("RRR"), (CLOUD_END_POS_X, CLOUD_END_POS_Y))
		self.sun_sprites.add( self.sun )

		self.cloud_sprites = pygame.sprite.OrderedUpdates()
		self.cloud_sprites.add( self.rain )

		self.currentDay = Day(self.sun)

		#Bug
		self.bug_sprites = pygame.sprite.OrderedUpdates()
		self.bug = BugSprite( MutateColor(0,0,0).sequence("RRR"), (300,550) )
		self.bug_sprites.add( self.bug )

		#self.flowers = list()
		#for row in range(0,self.grid.num_rows):
		#    for column in range(0,self.grid.num_columns):
		#        flower = Flower(self)
		#        self.grid.setNode(row, column, flower)
		#        self.flowers.append(flower)

		#self.Sprites.add( self.flowers )

		self.init_days()

	def init_days(self):
		levelDNA = list()
		for i in range(20):
			nucleobaseChoice = random.choice( game_screen.NUCLEOBASES ) 
			for time in range(random.randrange(13,18)):#(7,14)):
				levelDNA.append( nucleobaseChoice )
			if random.randrange(5) == 0:
				for time in range(random.randrange(13,18)):#(7,14)):
					levelDNA.append( "R" )
					levelDNA.append( "G" )
					levelDNA.append( "B" )

		#print levelDNA
		#print zip(*[levelDNA[i::3] for i in range(3)]) 
		for gene in zip(*[levelDNA[i::3] for i in range(3)]): 
			choose_weather_type = random.randrange(4)
			weather = self.sun
			if choose_weather_type < 3:
				weather = self.sun
			else:
				weather = self.rain
			day = Day(weather)
			day.setWeatherColor( MutateColor(0,0,0).sequence(gene) )
			self.days.append(day)
		#COLOR_INCREMENT = 85
		#sun_color = pygame.Color(255,0,0)
		#sun_color_time = 4
		#color_selector = 1
		#for i in range(31):
		#    day = Day(Sun())
		#    
		#    if color_selector == 1:
		#        sun_color.r = (sun_color.r + COLOR_INCREMENT) % 255
		#    elif color_selector == 2:
		#        sun_color.g = (sun_color.g + COLOR_INCREMENT) % 255
		#    elif color_selector == 3:
		#        sun_color.b = (sun_color.g + COLOR_INCREMENT) % 255

		#    if( sun_color == (0,0,0) ):
		#        if color_selector == 1:
		#            sun_color.r = COLOR_INCREMENT
		#        elif color_selector == 2:
		#            sun_color.g = COLOR_INCREMENT
		#        elif color_selector == 3:
		#            sun_color.b = COLOR_INCREMENT

		#    weather_red = sun_color.r
		#    weather_green = sun_color.g
		#    weather_blue = sun_color.b

		#    if( sun_color_time == 0):
		#        sun_color_time = random.randrange(1,5)
		#        color_selector = random.randrange(1,4)

		#    sun_color_time -= 1

		#    day.setWeatherColor( pygame.Color(weather_red, weather_green, weather_blue ) )
		#    self.days.append(day)
		
	def weather_start_done(self):
		self.state = CHANGE_DNA #BUG_BONUS
		self.end_turn()
			
	def weather_end_done(self):
		self.state = DAY_CHANGES
		self.end_turn()

	def bug_done(self):
		self.biggestBloomFlower.pollinate()
		#self.numFlowersSpawn = self.biggestBloomFlower.pollinationBonus + 1
		self.flower_bonus_done()
		#if self.biggestBloomFlower.player == PLAYER:
		#	self.currentPlayerFlower = self.spawnFlowerProgression( self.biggestBloomFlower.dna, 
		#															self.currentPlayerFlower,
		#														    self.flower_bonus_done)
		#	for x in range( self.biggestBloomFlower.pollinationBonus ):
		#		self.currentPlayerFlower = self.spawnFlowerProgression( self.biggestBloomFlower.dna, 
		#																self.currentPlayerFlower,
		#																self.flower_bonus_done)
		#else:
		#	self.currentRivalFlower = self.spawnFlowerProgression( self.biggestBloomFlower.dna, 
		#															self.currentRivalFlower,
		#															self.flower_bonus_done)
		#	for x in range( self.biggestBloomFlower.pollinationBonus ):
		#		self.currentRivalFlower = self.spawnFlowerProgression( self.biggestBloomFlower.dna, 
		#																self.currentRivalFlower,
		#																self.flower_bonus_done)

		#self.spawnFlower( self.biggestBloomFlower.dna, self.biggestBloomFlower.player, self.flower_bonus_done )
		#for x in range( self.biggestBloomFlower.pollinationBonus ):
		#	self.spawnFlower( self.biggestBloomFlower.dna, self.biggestBloomFlower.player, self.flower_bonus_done )

	def flower_bonus_done(self):
		#self.numFlowersSpawn -= 1
		#if self.numFlowersSpawn == 0:
		self.state = CHANGE_DNA
		self.end_turn()

	def flower_done(self):
		self.state = WEATHER_EFFECT_END
		self.end_turn()

	def isAbleToSpawn(self):
		node = self.grid.findRandomEmptySurroundingNode(self.currentPlayerFlower.root_position_index_x,
														self.currentPlayerFlower.root_position_index_y)
		if node:
			return True
		else:
			return False

	def end_turn(self):
		#WEATHER_EFFECT_START = 1
		#CHANGE_DNA = 2
		#SPAWN_PLANTS = 3
		#BUG_BONUS = 4
		#WEATHER_EFFECT_END = 5
		#DAY_CHANGES = 6
		#print "State: " + str(self.state)

		if self.state == WEATHER_EFFECT_START:
			if self.currentDay.weather.isSun:
				self.currentDay.weather.moveTo(SUN_START_POS_X, SUN_START_POS_Y, 0.5, self.fps, self.weather_start_done)
			else:
				self.currentDay.weather.moveTo(CLOUD_START_POS_X, CLOUD_START_POS_Y, 0.5, self.fps, self.weather_start_done)
			#self.sun.moveTo(300, 50, 0.5, self.fps, self.sun_start_done)
			#self.sun.scaleTo( 50, 50, 0.5, self.fps )
			#self.sun.fadeTo(255, 0.5, self.fps)
			self.currentDay.affect_flowers( self.flowers )
		elif self.state == BUG_BONUS:
			self.biggestBloomFlower = None
			for flower in self.flowers:
				if flower.bloomColor == self.currentDay.color:
					if self.biggestBloomFlower == None:
						self.biggestBloomFlower = flower
					if flower.bloomRadius > self.biggestBloomFlower.bloomRadius:
						self.biggestBloomFlower = flower
			if self.biggestBloomFlower != None:
				#print self.biggestBloomFlower.root_position
				self.bug.moveTo( self.biggestBloomFlower.root_position.centerx,
								 self.biggestBloomFlower.root_position.centery,
								 1, self.fps, self.bug_done  )
			else:
				self.state = CHANGE_DNA
				self.end_turn()
		elif self.state == CHANGE_DNA:
			pass
		elif self.state == SPAWN_PLANTS:
			flower = self.spawnFlowerProgression( self.next_flower_dna, self.currentPlayerFlower )
			if flower:
				self.currentPlayerFlower = flower
				self.currentPlayerFlower.selected = True

			#self.spawnFlower( self.next_flower_dna, PLAYER ) #, self.flower_done )
			#AI
			#CHOOSE_ROTATE_LEFT = 0
			#CHOOSE_ROTATE_RIGHT = 1
			#CHOOSE_SWAP_1 = 2
			#CHOOSE_SWAP_2 = 3
			#CHOOSE_RANDOM = 4
			chooseAction = random.randrange(5)
			if chooseAction == CHOOSE_ROTATE_LEFT:
				self.rival_next_flower_dna.rotate_left()
				#print "LEFT"
			elif chooseAction == CHOOSE_ROTATE_RIGHT:
				self.rival_next_flower_dna.rotate_right()
				#print "RIGHT"
			elif chooseAction == CHOOSE_SWAP_1:
				self.rival_next_flower_dna.swap_pair_1()
				#print "SWAP 1"			
			elif chooseAction == CHOOSE_SWAP_2:
				self.rival_next_flower_dna.swap_pair_2()
				#print "SWAP 2"
			elif chooseAction == CHOOSE_RANDOM:
				self.rival_next_flower_dna.random_mutation( list(NUCLEOBASES) )
				#print "RANDOM"

			spawnflower = self.spawnFlowerProgression(	self.rival_next_flower_dna, 
														self.currentRivalFlower,
														self.flower_done )
			if spawnflower:
				self.currentRivalFlower = spawnflower
			else:
				node = None
				for flower in self.flowers:
					if flower.player == RIVAL:
						node = self.grid.findNodeWithEmpty(flower.root_position_index_x,
														   flower.root_position_index_y)
						if node:
							break

				if node:
					spawnflower = self.spawnFlowerProgression(	self.rival_next_flower_dna, 
																node.flower,
																self.flower_done )
					if spawnflower:
						self.currentRivalFlower = spawnflower
				else:
					print "GAME OVER"
					self.is_completed = True
					return

			#spawnflower = None
			#while spawnflower == None:
			#	spawnflower = self.spawnFlowerProgression(	self.rival_next_flower_dna, 
			#											self.currentRivalFlower,
			#											self.flower_done )
				#if spawnflower:
				#	self.currentRivalFlower = spawnflower
				#else:
					#for flower in self.flowers:
					#	node = None
					#	if flower.player == RIVAL:
					#		node = self.grid.findRandomEmptySurroundingNode(flower.root_position_index_x,
					#														flower.root_position_index_y)
					#	if node:
					#		self.currentRivalFlower = flower
					#		break

			#self.spawnFlower( self.rival_next_flower_dna, RIVAL, self.flower_done )
		elif self.state == WEATHER_EFFECT_END:
			if self.currentDay.weather.isSun:
				self.currentDay.weather.moveTo(SUN_END_POS_X, SUN_END_POS_Y, 0.5, self.fps, self.weather_end_done)
			else:
				self.currentDay.weather.moveTo(CLOUD_END_POS_X, CLOUD_END_POS_Y, 0.5, self.fps, self.weather_end_done)
			#self.sun.moveTo(0,50, 0.5, self.fps, self.sun_end_done)
			#self.sun.image.set_alpha(255)
			#self.sun.fadeTo(0, 0.5, self.fps)
		elif self.state == DAY_CHANGES:
			day = self.days.popleft()
			#print day.desc
			self.currentDay = day
			if len(self.days) == 0:
				print "GAME OVER"
				self.is_completed = True
				return

			#self.days.append( day )
			self.sun.setColor( self.currentDay.color )
			self.state = WEATHER_EFFECT_START
			self.numTurns += 1
			self.end_turn()

	def spawnFlowerProgression(self, dna, prevflower, callback=None):
		if self.grid.isThereVacantNode():

			node = self.grid.findRandomEmptySurroundingNode(prevflower.root_position_index_x,
															prevflower.root_position_index_y)
			if node:
				flower = Flower(self, prevflower.player)
				flower.setDNA( copy.copy(dna) )
				self.grid.setNode(	node.rowIndex, 
									node.columnIndex,
									flower )

				self.flowers.add( flower )
				width, height = flower.NORM_PLANT_SIZE
				flower.scaleTo( width, height, 1, self.fps, callback )
				return flower
		else:
			print "GAME OVER"
			self.is_completed = True

	def spawnFlower(self, dna, player, callback=None):
		if self.grid.isThereVacantNode():
				flower = Flower(self, player)
				flower.setDNA( copy.copy(dna) )

				nofindSpot = True
				while nofindSpot and self.grid.isThereVacantNode():
					nofindSpot = not self.grid.setNode( random.randrange(0,self.grid.num_rows),
								   random.randrange(0,self.grid.num_columns),
								   flower)

				self.flowers.add( flower )
				width, height = flower.NORM_PLANT_SIZE
				flower.scaleTo( width, height, 1, self.fps, callback )
		else:
			print "GAME OVER"
			self.is_completed = True

	def getNumFlowers(self):
		numPlayerFlowers = 0
		numRivalFlowers = 0
		for flower in self.flowers:
			if flower.player == PLAYER:
				numPlayerFlowers += 1
			elif flower.player == RIVAL:
				numRivalFlowers += 1
		return (numPlayerFlowers, numRivalFlowers)

	def flowerDied(self, flower):
		self.grid.rows[flower.root_position_index_x][flower.root_position_index_y].flower = None
		flower.kill()

	def begin_frame(self):
		self.clock.tick()
		self.fps = self.clock.get_fps()
		for flower in self.flowers:
			if flower.player == PLAYER:
				if pygame.mouse.get_pressed() == (1,0,0):
						if flower.rect.collidepoint(pygame.mouse.get_pos()):
							flower.mouseActive = True

				if flower.mouseActive == True:
					if pygame.mouse.get_pressed() == (0,0,0):
						flower.mouseActive = False
						if flower.rect.collidepoint(pygame.mouse.get_pos()):
							print "Clicked"
							flower.selected = True
							self.currentPlayerFlower = flower
							self.next_flower_dna = copy.copy(flower.dna)
							flower.redraw()
							continue

				if flower != self.currentPlayerFlower:
					flower.selected = False
					flower.redraw()

		self.flowers.update(self.fps)
		self.sun_sprites.update()
		self.cloud_sprites.update()
		self.bug_sprites.update()